/*
 *	Shared Object plugin for Snort Intrusion Detection System.
 *	Copyright (C) 2008 dxp
 *
 *	See file "COPYING" for TERMS AND CONDITIONS.
 */
/*
 * LOCAL Dynamic Rule
 */

#include <sf_snort_plugin_api.h>
#include <sf_dynamic_meta.h>
#include <sf_snort_packet.h>
#include <stdio.h>

int rule2532eval (void *p);

/*
 * Rule Options
 */

/* flow:established; */
static FlowFlags flow_opt_data = 
{
	FLOW_ESTABLISHED
};
static RuleOption flow_opt = { OPTION_TYPE_FLOWFLAGS, &flow_opt_data };

/* content:"MZ"; offset:0; */
static ContentInfo content_opt_data = 
{
	(u_int8_t *) "MZ",			// pattern
	0,			// depth
	0,			// offset
	CONTENT_BUF_NORMALIZED,	// flags
	NULL,			// internal use
	NULL,			// internal use
	0			// internal use
};
static RuleOption content_opt = { OPTION_TYPE_CONTENT, &content_opt_data };

/* move cursor back to start of "MZ" */
static CursorInfo cursor_mov_opt_data =
{
	-2,					// offset
	CONTENT_BUF_NORMALIZED | CONTENT_RELATIVE	// flags
};
static RuleOption cursor_mov_opt = { OPTION_TYPE_SET_CURSOR, &cursor_mov_opt_data };

static RuleOption *options[] = { &flow_opt, &content_opt, &cursor_mov_opt, NULL };

/*
 * References
 */

static RuleReference ref1 = { "url", "www.microsoft.com/whdc/system/platform/firmware/PECOFF.mspx" };
static RuleReference *references[] = { &ref1 };

/*
 * Rule Definition
 */

Rule rule2532 = 
{
	/* Rule Header */
	
	// alert tcp any any <> any any
	{
		IPPROTO_TCP,		// protocol
		ANY_NET,		// source ip
		ANY_PORT,		// source port
		RULE_BIDIRECTIONAL,	// direction, non-zero is bidirectional
		ANY_NET,		// destination ip
		ANY_PORT		// destination port
	},
	
	/* Rule metadata */
	
	// sid: 2532; rev: 1; classtype:"policy-violation"; msg:"LOCAL SO PE header";
	{
		3,			// generator id, 3 = dynamic rules
		2532,			// sid
		1,			// revision
		"policy-violation",	// classification
		0,			// priority
		"LOCAL SO PE header",	// message
		references		// NULL terminated array of references
	},
	
	options,			// rule options
	&rule2532eval,			// rule eval function
	0,				// rule initialized (used internally)
	0,				// rule option count (used internally)
	0				// (used internally)
};

/*
 * Evaluation Rule
 */

int rule2532eval (void *p)
{
	SFSnortPacket *sp = (SFSnortPacket *) p;
	/* data types here must match pointer typecasts within the rule */
	u_int32_t off = 0;
	u_int32_t pe = 0;
	u_int32_t bin_size = 0;
	const u_int8_t *cursor = 0;
	
	/* check for established flow */
	if ( checkFlow(p, options[0]->option_u.flowFlags) == RULE_MATCH )
		/* check for start MZ header */
		if ( contentMatch(p, options[1]->option_u.content, &cursor) == CONTENT_MATCH )
			/* move cursor back 2 chars to start of MZ header */
			if ( setCursor(p, options[2]->option_u.cursor, &cursor) == CURSOR_IN_BOUNDS ) {
			
					/* compute size of binary (exe) buffer, excluding any headers */
					bin_size = sp->payload_size - (cursor - sp->payload);
					
					/* make sure there's enough buffer to read PE offset */
					if ( ! (bin_size >= (0x3c + sizeof(off))) )
						return RULE_NOMATCH;
					
					/* get offset to PE header and make sure it's within the buffer */
					off = * (u_int32_t *) (cursor + 0x3C);
					if ( bin_size < (off + sizeof(pe)) )
						return RULE_NOMATCH;
						
					/* read value of PE magic and make sure it's correct */
					pe = * (u_int32_t *) (cursor + off);
					if (pe == 0x00004550)
						return RULE_MATCH;
			}
		
	return RULE_NOMATCH;
}
